import { decorate, observable } from "mobx";
import { observer } from "mobx-react";
import * as React from "react";
import { RootStore } from "src/core/store";
import { Base } from "./Base";

export type ActionProviderOptions<TAction, TProps> = {
	createAction: (component: Base<TAction, TProps>, rootStore: RootStore) => TAction;
	onUnmount: (component: Base<TAction, TProps>, rootStore: RootStore) => void;
	onPropsChange: (component: Base<TAction, TProps>, nextProps: TProps) => void;
};

export type ActionProviderProps<TAction> = {
	children: (action: TAction) => React.ReactNode;
};

export type ActionProviderState<TAction, TProps> = { _this: Base<TAction, TProps> };

export function createActionProvider<TAction, TProps extends ActionProviderProps<TAction>>(
	options: ActionProviderOptions<TAction, TProps>
) {
	const { Provider: ActionContextProvider, Consumer: ActionContextConsumer } = React.createContext<TAction>(
		undefined as any
	);

	let ActionProvider = class CreateEntityActionProvider extends Base<TAction, TProps> {
		public static getDerivedStateFromProps(
			nextProps: Readonly<TProps>,
			prevState: ActionProviderState<TAction, TProps>
		): Partial<ActionProviderState<TAction, TProps>> | null {
			const { _this } = prevState;
			options.onPropsChange(_this, nextProps);

			return null;
		}

		public state = { _this: this };

		constructor(props: TProps) {
			super(props);

			this.action = options.createAction(this, this.rootStore);
		}

		public componentWillUnmount(): void {
			options.onUnmount(this, this.rootStore);
		}

		public render(): React.ReactNode {
			const { props, action } = this;

			return (
				<ActionContextProvider value={action}>
					{typeof props.children === "function" && props.children(action)}
				</ActionContextProvider>
			);
		}
	};

	decorate(ActionProvider, {
		action: observable
	});
	ActionProvider = observer(ActionProvider);

	return { ActionProvider, ActionContextProvider, ActionContextConsumer };
}
