import { InMemoryCache } from "apollo-cache-inmemory";
import { ApolloClient, ApolloError, ApolloQueryResult, MutationOptions, QueryOptions } from "apollo-client";
import { ApolloLink, execute, FetchResult } from "apollo-link";
import { onError } from "apollo-link-error";
import { createUploadLink } from "apollo-upload-client";
import * as _ from "lodash";

// region typeDef
export type BaseRequest = {};
export type BaseQueryRequest = BaseRequest;
export type BaseMutationRequest = BaseRequest;

export type SuccessfulQueryResponse<TData> = {
	success: true;
	rawResponse: ApolloQueryResult<TData>;
};
export type FailedQueryResponse = {
	success: false;
	rawResponse: ApolloError;
};

export type SuccessfulMutationResponse<TData> = {
	success: true;
	rawResponse: FetchResult<TData>;
};
export type FailedMutationResponse = {
	success: false;
	rawResponse: ApolloError;
};
// endregion

const parseHeaders = (rawHeaders: string) => {
	const headers = new Headers();
	// Replace instances of \r\n and \n followed by at least one space or horizontal tab with a space
	// https://tools.ietf.org/html/rfc7230#section-3.2
	const preProcessedHeaders = rawHeaders.replace(/\r?\n[\t ]+/g, " ");
	preProcessedHeaders.split(/\r?\n/).forEach(line => {
		const parts = line.split(":");
		const key = parts.shift()!.trim();
		if (key) {
			const value = parts.join(":").trim();
			headers.append(key, value);
		}
	});
	return headers;
};

const customFetch: (context: any) => ((input: RequestInfo, init?: RequestInit) => Promise<Response>) = context => (
	url,
	options = {}
) =>
	new Promise((resolve, reject) => {
		const xhr = new XMLHttpRequest();
		const { headers, onProgress } = context;

		xhr.onload = () => {
			const opts = {
				status: xhr.status,
				statusText: xhr.statusText,
				headers: parseHeaders(xhr.getAllResponseHeaders() || "")
			};

			// @ts-ignore
			opts.url = "responseURL" in xhr ? xhr.responseURL : opts.headers.get("X-Request-URL");
			// @ts-ignore
			const body = "response" in xhr ? xhr.response : xhr.responseText;
			resolve(new Response(body, opts));
		};

		xhr.onerror = () => {
			xhr.abort();
			reject(new TypeError("Network request failed"));
		};

		xhr.ontimeout = () => {
			reject(new TypeError("Network request failed"));
		};

		xhr.onabort = () => {
			reject(new TypeError("Network request failed"));
		};

		// @ts-ignore
		xhr.open(options.method, url, true);

		// @ts-ignore
		Object.keys(options.headers).forEach(key => {
			// @ts-ignore
			xhr.setRequestHeader(key, options.headers[key]);
		});

		if (!_.isNil(headers))
			Object.keys(headers).forEach(key => {
				xhr.setRequestHeader(key, headers[key]);
			});

		if (xhr.upload) xhr.upload.onprogress = onProgress;

		xhr.send(options.body);
	});

export class Base {
	public static client = new ApolloClient({
		link: ApolloLink.from([
			onError((/*{ graphQLErrors, networkError }*/) => {
				// if (graphQLErrors)
				// 	graphQLErrors.map(({ message, locations, path }) =>
				// 		console.log(`[GraphQL error]: Message: ${message}, Location: ${locations}, Path: ${path}`)
				// 	);
				// if (networkError) console.log(`[Network error]: ${networkError}`);
			}),
			new ApolloLink(operation => {
				operation.setContext((context: any = {}) => {
					const headers = { ...context.headers };

					if (context.token) headers["x-token"] = context.token;

					return {
						...context,
						headers
					};
				});

				return execute(
					createUploadLink({
						uri: "https://open.fing.edu.uy/api/v1/graphql",
						credentials: "same-origin",
						headers: {
							Accept: "application/json"
						},
						fetch: customFetch(operation.getContext())
					}),
					operation
				);
			})
		]),
		cache: new InMemoryCache(),
		defaultOptions: {
			query: {
				fetchPolicy: "no-cache"
			}
		}
	});

	public static query<TData, TVariables, TRequest>(
		request: TRequest,
		options: QueryOptions<TVariables>
	): Promise<SuccessfulQueryResponse<TData> | FailedQueryResponse> {
		return this.client
			.query<TData, TVariables>(options)
			.then(response => this.processResponse(request, response))
			.catch((reason: ApolloError) => this.processErrorResponse(request, reason));
	}

	public static mutate<TData, TVariables, TRequest>(
		request: TRequest,
		options: MutationOptions<any, TVariables>
	): Promise<SuccessfulMutationResponse<TData> | FailedMutationResponse> {
		return this.client
			.mutate(options)
			.then(response => this.processResponse(request, response))
			.catch((reason: ApolloError) => this.processErrorResponse(request, reason));
	}

	public static processResponse<TData, TRequest>(
		request: TRequest,
		result: ApolloQueryResult<TData>
	): SuccessfulQueryResponse<TData>;
	public static processResponse<TData, TRequest>(
		request: TRequest,
		result: FetchResult<TData>
	): SuccessfulMutationResponse<TData>;
	public static processResponse<TData, TRequest>(
		request: TRequest,
		result: ApolloQueryResult<TData> | FetchResult<TData>
	): SuccessfulQueryResponse<TData> | SuccessfulMutationResponse<TData> {
		return {
			success: true,
			rawResponse: result
		};
	}

	public static processErrorResponse<TRequest extends BaseQueryRequest>(
		request: TRequest,
		result: ApolloError
	): FailedQueryResponse;
	public static processErrorResponse<TRequest extends BaseMutationRequest>(
		request: TRequest,
		result: ApolloError
	): FailedMutationResponse;
	public static processErrorResponse<TRequest = BaseQueryRequest | BaseMutationRequest>(
		request: TRequest,
		result: ApolloError
	): FailedQueryResponse | FailedMutationResponse {
		return {
			success: false,
			rawResponse: result
		};
	}

	// public static processError(graphQLError: GraphQLError): {code: string, message:string} | undefined{
	//
	// }
}
