import { ReactNativeFile } from "apollo-upload-client";
import gql from "graphql-tag";
import * as _ from "lodash";
import * as Errors from "../errors";
import * as Factories from "../factories";
import * as Models from "../models";
import { Base, BaseQueryRequest } from "./Base";

// region FetchCourses
export type FetchCoursesRequest = BaseQueryRequest;
export type FetchCoursesData = {
	courses: Array<{ id: number; code: string; name?: string; eva?: string; iconURL?: string }>;
};
export type FetchCoursesVariables = {};
export type SuccessfulFetchCoursesResponse = {
	success: true;
	courses: Models.Course[];
};
export type FailedFetchCoursesResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchCoursesResponse = SuccessfulFetchCoursesResponse | FailedFetchCoursesResponse;
// endregion

// region FetchCourseByCode
export type FetchCourseByCodeRequest = { code: string };
export type FetchCourseByCodeData = {
	courseByCode: {
		id: number;
		code: string;
		name?: string;
		eva?: string;
		semester?: number;
		year?: number;
		classes: Array<{ id: number; number?: number; title?: string; videoURL: string }>;
	};
};
export type FetchCourseByCodeVariables = { code: string };
export type SuccessfulFetchCourseByCodeResponse = {
	success: true;
	course: Models.Course;
};
export type FailedFetchCourseByCodeResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchCourseByCodeResponse = SuccessfulFetchCourseByCodeResponse | FailedFetchCourseByCodeResponse;
// endregion

// region CreateCourse
export type CreateCourseRequest = {
	token: string;
	code: string;
	name: string;
	eva?: string;
	semester: number;
	year: number;
	icon?: File | ReactNativeFile;
	onProgress?: (e: ProgressEvent) => void;
};
export type CreateCourseData = {
	createCourse: {
		id: number;
		code: string;
		name?: string;
		eva?: string;
		iconURL?: string;
		classes: [
			{
				id: number;
				number: number;
				title?: string;
				videoURL?: string;
			}
		];
	};
};
export type CreateCourseVariables = {
	code: string;
	name: string;
	eva: string | undefined;
	semester: number;
	year: number;
	icon: File | ReactNativeFile | undefined;
};
export type SuccessfulCreateCourseResponse = {
	success: true;
	course: Models.Course;
};
export type FailedCreateCourseResponse = {
	success: false;
	error: Errors.GenericError;
};
export type CreateCourseResponse = SuccessfulCreateCourseResponse | FailedCreateCourseResponse;
// endregion

// region UpdateCourse
export type UpdateCourseRequest = {
	token: string;
	id: number;
	code?: string;
	name?: string;
	eva?: string;
	semester?: number;
	year?: number;
	icon?: File | ReactNativeFile;
	onProgress?: (e: ProgressEvent) => void;
};
export type UpdateCourseData = {
	updateCourse: {
		id: number;
		code: string;
		name?: string;
		eva?: string;
		classes: Array<{
			id: number;
			number?: number;
			title?: string;
			videoURL?: string;
		}>;
	};
};
export type UpdateCourseVariables = {
	id: number;
	code?: string;
	name?: string;
	eva?: string;
	semester?: number;
	year?: number;
	icon?: File | ReactNativeFile;
};
export type SuccessfulUpdateCourseResponse = {
	success: true;
	course: Models.Course;
};
export type FailedUpdateCourseResponse = {
	success: false;
	error: Errors.GenericError;
};
export type UpdateCourseResponse = SuccessfulUpdateCourseResponse | FailedUpdateCourseResponse;
// endregion

// region DeleteCourse
export type DeleteCourseRequest = {
	token: string;
	courseCode: string;
};
export type DeleteCourseData = { deleteCourseByCode: boolean };
export type DeleteCourseVariables = { courseCode: string };
export type SuccessfulDeleteCourseResponse = {
	success: true;
	deleted: boolean;
};
export type FailedDeleteCourseResponse = {
	success: false;
	error: Errors.GenericError;
};
export type DeleteCourseResponse = SuccessfulDeleteCourseResponse | FailedDeleteCourseResponse;
// endregion

export class Course extends Base {
	public static async fetchCourses(request: FetchCoursesRequest = {}): Promise<FetchCoursesResponse> {
		const apiResponse = await this.query<FetchCoursesData, FetchCoursesVariables, FetchCoursesRequest>(request, {
			query: gql`
				{
					courses {
						id
						code
						name
						eva
						iconURL
					}
				}
			`,
			variables: {}
		});

		if (!apiResponse.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener la lista de cursos.")
			};

		const { data } = apiResponse.rawResponse;

		if (!_.isArray(data.courses))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener la lista de cursos.")
			};

		return {
			success: true,
			courses: data.courses.map((course: any) => Factories.Course.fromJSON(course))
		};
	}

	public static async fetchCourseByCode(request: FetchCourseByCodeRequest): Promise<FetchCourseByCodeResponse> {
		const { code } = request;

		const apiResponse = await this.query<
			FetchCourseByCodeData,
			FetchCourseByCodeVariables,
			FetchCourseByCodeRequest
		>(request, {
			query: gql`
				query courseByCode($code: String!) {
					courseByCode(code: $code) {
						id
						code
						name
						eva
						semester
						year
						iconURL
						classes {
							id
							number
							title
							videoURL
						}
					}
				}
			`,
			variables: { code }
		});

		if (!apiResponse.success) {
			const { graphQLErrors } = apiResponse.rawResponse;

			let error;
			if (graphQLErrors.length !== 0) {
				const graphQLError = graphQLErrors[0];

				if (graphQLError.name === "OBJECT_NOT_FOUND")

				console.log(graphQLErrors[0]);
			}

			if (error === undefined) error = Errors.genericError("Hubo un error al intentar obtener el curso.");

			return {
				success: false,
				error
			};
		}

		const { data } = apiResponse.rawResponse;

		if (_.isNil(data) || _.isNil(data.courseByCode))
			return { success: false, error: Errors.genericError("Hubo un error al intentar obtener el curso.") };

		return {
			success: true,
			course: Factories.Course.fromJSON(data.courseByCode)
		};
	}

	public static async createCourse(request: CreateCourseRequest): Promise<CreateCourseResponse> {
		const { token, code, name, eva, semester, year, icon, onProgress } = request;

		const response = await this.mutate<CreateCourseData, CreateCourseVariables, CreateCourseRequest>(request, {
			mutation: gql`
				mutation createCourse(
					$code: String!
					$name: String!
					$eva: String
					$semester: Int!
					$year: Int!
					$icon: Upload
				) {
					createCourse(
						course: { code: $code, name: $name, eva: $eva, semester: $semester, year: $year, icon: $icon }
					) {
						id
						code
						name
						eva
						iconURL
						classes {
							id
							number
							title
							videoURL
						}
					}
				}
			`,
			variables: { code, name, eva, semester, year, icon },
			context: { token, onProgress }
		});

		if (!response.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar crear el curso.")
			};

		const { data } = response.rawResponse;

		if (_.isNil(data) || _.isNil(data.createCourse))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar crear el curso.")
			};

		return {
			success: true,
			course: Factories.Course.fromJSON(data.createCourse)
		};
	}

	public static async updateCourse(request: UpdateCourseRequest): Promise<UpdateCourseResponse> {
		const { token, id, code, name, eva, semester, year, icon, onProgress } = request;

		const queryResponse = await this.mutate<UpdateCourseData, UpdateCourseVariables, UpdateCourseRequest>(request, {
			mutation: gql`
				mutation updateCourse(
					$id: Int!
					$code: String
					$name: String
					$eva: String
					$semester: Int
					$year: Int
					$icon: Upload
				) {
					updateCourse(
						id: $id
						values: { code: $code, name: $name, eva: $eva, semester: $semester, year: $year, icon: $icon }
					) {
						id
						code
						name
						eva
						classes {
							id
							number
							title
							videoURL
						}
					}
				}
			`,
			variables: { id, code, name, eva, semester, year, icon },
			context: { token, onProgress }
		});

		if (!queryResponse.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar actualizar el curso.")
			};

		const { data } = queryResponse.rawResponse;

		if (_.isNil(data) || _.isNil(data.updateCourse))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar actualizar el curso.")
			};

		return {
			success: true,
			course: Factories.Course.fromJSON(data.updateCourse)
		};
	}

	public static async deleteCourse(request: DeleteCourseRequest): Promise<DeleteCourseResponse> {
		const { token, courseCode } = request;

		const apiResponse = await this.mutate<DeleteCourseData, DeleteCourseVariables, DeleteCourseRequest>(request, {
			mutation: gql`
				mutation deleteCourseByCode($courseCode: String!) {
					deleteCourseByCode(courseCode: $courseCode)
				}
			`,
			variables: { courseCode },
			context: { token }
		});

		if (
			!apiResponse.success ||
			_.isNil(apiResponse.rawResponse.data) ||
			_.isNil(apiResponse.rawResponse.data.deleteCourseByCode)
		)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar eliminar el curso.")
			};

		const { data } = apiResponse.rawResponse;

		return {
			success: true,
			deleted: data.deleteCourseByCode
		};
	}
}
