import gql from "graphql-tag";
import * as _ from "lodash";
import * as Errors from "../errors";
import * as Factories from "../factories";
import * as Models from "../models";
import { Base } from "./Base";

// region FetchCourseClass
export type FetchCourseClassRequest = {
	courseCode: string;
	classNo: number;
};
export type FetchCourseClassData = {
	courseClassByClassNo: {
		id: number;
		number: number;
		title?: string;
		videoURL?: string;
		disabled?: boolean
	};
};
export type FetchCourseClassVariables = {
	courseCode: string;
	classNo: number;
};
export type SuccessfulFetchCourseClassResponse = {
	success: true;
	courseClass: Models.CourseClass;
};
export type FailedFetchCourseClassResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchCourseClassResponse = SuccessfulFetchCourseClassResponse | FailedFetchCourseClassResponse;
// endregion

// region FetchLatestCourseClasses
export type FetchLatestCourseClassesRequest = {};
export type FetchLatestCourseClassesData = {
	latestCourseClasses: Array<{
		id: number;
		number: number;
		title?: string;
		videoURL?: string;
		createdAt?: string;
	}>;
};
export type FetchLatestCourseClassesVariables = {};
export type SuccessfulFetchLatestCourseClassesResponse = {
	success: true;
	courseClasses: Models.CourseClass[];
};
export type FailedFetchLatestCourseClassesResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchLatestCourseClassesResponse =
	| SuccessfulFetchLatestCourseClassesResponse
	| FailedFetchLatestCourseClassesResponse;
// endregion

// region CreateCourseClass
export type CreateCourseClassRequest = {
	token: string;
	number: number;
	title: string;
	videoURL?: string;
	disabled?: boolean;
	courseId: number;
};
export type CreateCourseClassData = {
	createCourseClass: {
		id: number;
		number: number;
		title?: string;
		videoURL?: string;
	};
};
export type CreateCourseClassVariables = {
	number: number;
	title: string;
	videoURL?: string;
	disabled?: boolean;
	courseId: number;
};
export type SuccessfulCreateCourseClassResponse = {
	success: true;
	courseClass: Models.CourseClass;
};
export type FailedCreateCourseClassResponse = {
	success: false;
	error: Errors.GenericError | Errors.ObjectNotFoundError;
};
export type CreateCourseClassResponse = SuccessfulCreateCourseClassResponse | FailedCreateCourseClassResponse;
// endregion

// region UpdateCourseClass
export type UpdateCourseClassRequest = {
	token: string;
	id: number;
	number?: number;
	title?: string;
	videoURL?: string;
	disabled?: boolean;
};
export type UpdateCourseClassData = {
	updateCourseClass: {
		id: number;
		number: number;
		title?: string;
		videoURL?: string;
	};
};
export type UpdateCourseClassVariables = {
	id: number;
	number?: number;
	title?: string;
	videoURL?: string;
	disabled?: boolean;
};
export type SuccessfulUpdateCourseClassResponse = {
	success: true;
	courseClass: Models.CourseClass;
};
export type FailedUpdateCourseClassResponse = {
	success: false;
	error: Errors.GenericError;
};
export type UpdateCourseClassResponse = SuccessfulUpdateCourseClassResponse | FailedUpdateCourseClassResponse;
// endregion

// region DeleteCourseClass
export type DeleteCourseClassRequest = {
	token: string;
	id: number;
};
export type DeleteCourseClassData = { deleteCourseClass: boolean };
export type DeleteCourseClassVariables = {
	id: number;
};
export type SuccessfulDeleteCourseClassResponse = {
	success: true;
	deleted: boolean;
};
export type FailedDeleteCourseClassResponse = {
	success: false;
	error: Errors.GenericError;
};
export type DeleteCourseClassResponse = SuccessfulDeleteCourseClassResponse | FailedDeleteCourseClassResponse;
// endregion

export class CourseClass extends Base {
	public static async fetchCourseClass(request: FetchCourseClassRequest): Promise<FetchCourseClassResponse> {
		const { courseCode, classNo } = request;

		const apiResponse = await this.query<FetchCourseClassData, FetchCourseClassVariables, FetchCourseClassRequest>(
			request,
			{
				query: gql`
					query fetchCourseClass($courseCode: String!, $classNo: Int!) {
						courseClassByClassNo(courseCode: $courseCode, classNo: $classNo) {
							id
							number
							title
							videoURL
							disabled
						}
					}
				`,
				variables: { courseCode, classNo }
			}
		);

		if (
			!apiResponse.success ||
			_.isNil(apiResponse.rawResponse.data) ||
			_.isNil(apiResponse.rawResponse.data.courseClassByClassNo)
		)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener la clase.")
			};

		const { data } = apiResponse.rawResponse;

		return {
			success: true,
			courseClass: Factories.CourseClass.fromJSON(data.courseClassByClassNo)
		};
	}

	public static async fetchLatestCourseClasses(
		request: FetchLatestCourseClassesRequest = {}
	): Promise<FetchLatestCourseClassesResponse> {
		const apiResponse = await this.query<
			FetchLatestCourseClassesData,
			FetchLatestCourseClassesVariables,
			FetchLatestCourseClassesRequest
		>(request, {
			query: gql`
				{
					latestCourseClasses {
						id
						title
						number
						course {
							id
							code
							name
							iconURL
						}
						createdAt
					}
				}
			`
		});

		if (
			!apiResponse.success ||
			_.isNil(apiResponse.rawResponse.data) ||
			_.isNil(apiResponse.rawResponse.data.latestCourseClasses)
		)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener la clase.")
			};

		const { data } = apiResponse.rawResponse;

		return {
			success: true,
			courseClasses: data.latestCourseClasses.map(courseClass => Factories.CourseClass.fromJSON(courseClass))
		};
	}

	public static async createCourseClass(request: CreateCourseClassRequest): Promise<CreateCourseClassResponse> {
		const { token } = request;

		const response = await this.mutate<CreateCourseClassData, CreateCourseClassVariables, CreateCourseClassRequest>(
			request,
			{
				mutation: gql`
					mutation createCourseClass(
						$number: Int!
						$title: String!
						$videoURL: String!
						$disabled: Boolean
						$courseId: Int!
					) {
						createCourseClass(
							courseClass: {
								number: $number
								title: $title
								videoURL: $videoURL
								disabled: $disabled
								courseId: $courseId
							}
						) {
							id
							number
							title
							videoURL
							disabled
						}
					}
				`,
				variables: request,
				context: { token }
			}
		);

		if (!response.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar crear la clase.")
			};

		const { data } = response.rawResponse;

		if (_.isNil(data) || _.isNil(data.createCourseClass))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar crear la clase.")
			};

		return {
			success: true,
			courseClass: Factories.CourseClass.fromJSON(data.createCourseClass)
		};
	}

	public static async updateCourseClass(request: UpdateCourseClassRequest): Promise<UpdateCourseClassResponse> {
		const { token, id, number, title, videoURL, disabled } = request;

		const response = await this.mutate<UpdateCourseClassData, UpdateCourseClassVariables, UpdateCourseClassRequest>(
			request,
			{
				mutation: gql`
					mutation updateCourseClass(
						$id: Int!
						$number: Int
						$title: String
						$videoURL: String
						$disabled: Boolean
					) {
						updateCourseClass(
							id: $id
							values: { number: $number, title: $title, videoURL: $videoURL, disabled: $disabled }
						) {
							id
							number
							title
							videoURL
							disabled

							course {
								id
							}

							createdAt
							createdBy {
								id
								name
							}

							updatedAt
							updatedBy {
								id
								name
							}
						}
					}
				`,
				variables: { id, number, title, videoURL, disabled },
				context: { token }
			}
		);

		if (!response.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar actualizar la clase.")
			};

		const { data } = response.rawResponse;

		if (_.isNil(data) || _.isNil(data.updateCourseClass))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar actualizar la clase.")
			};

		return {
			success: true,
			courseClass: Factories.CourseClass.fromJSON(data.updateCourseClass)
		};
	}

	public static async deleteCourseClass(request: DeleteCourseClassRequest): Promise<DeleteCourseClassResponse> {
		const { token, id } = request;

		const apiResponse = await this.mutate<
			DeleteCourseClassData,
			DeleteCourseClassVariables,
			DeleteCourseClassRequest
		>(request, {
			mutation: gql`
				mutation deleteCourseClass($id: Int!) {
					deleteCourseClass(id: $id)
				}
			`,
			variables: { id },
			context: { token }
		});

		if (
			!apiResponse.success ||
			_.isNil(apiResponse.rawResponse.data) ||
			_.isNil(apiResponse.rawResponse.data.deleteCourseClass)
		)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar eliminar la clase.")
			};

		const { data } = apiResponse.rawResponse;

		return {
			success: true,
			deleted: data.deleteCourseClass
		};
	}
}
