import gql from "graphql-tag";
import * as _ from "lodash";
import * as Errors from "../errors";
import * as Factories from "../factories";
import * as Models from "../models";
import { Base } from "./Base";

// region FetchFAQ
export type FetchFAQRequest = {
	id: number;
};
export type FetchFAQData = {
	faq: {
		id: number;
		title?: string;
		content?: string;
		isHTML?: boolean;

		createdAt?: string;
		createdBy?: {
			id: number;
			name?: string;
		};

		updatedAt?: string;
		updatedBy?: {
			id: number;
			name?: string;
		};
	};
};
export type FetchFAQVariables = {
	id: number;
};
export type SuccessfulFetchFAQResponse = {
	success: true;
	faq: Models.FAQ;
};
export type FailedFetchFAQResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchFAQResponse = SuccessfulFetchFAQResponse | FailedFetchFAQResponse;
// endregion

// region FetchFAQs
export type FetchFAQsRequest = {};
export type FetchFAQsData = {
	faqs: Array<{
		id: number;
		title?: string;
		content?: string;
		isHTML?: boolean;

		createdAt?: string;
		createdBy?: {
			id: number;
			name?: string;
		};
	}>;
};
export type FetchFAQsVariables = {};
export type SuccessfulFetchFAQsResponse = {
	success: true;
	faqs: Models.FAQ[];
};
export type FailedFetchFAQsResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchFAQsResponse = SuccessfulFetchFAQsResponse | FailedFetchFAQsResponse;
// endregion

// region CreateFAQ
export type CreateFAQRequest = {
	token: string;
	title: string;
	content: string;
	isHTML: boolean;
};
export type CreateFAQData = {
	createFAQ: {
		id: number;
		title?: string;
		content?: string;
		isHTML?: boolean;

		createdAt?: string;
		createdBy?: {
			id: number;
			name?: string;
		};

		updatedAt?: string;
		updatedBy?: {
			id: number;
			name?: string;
		};
	};
};
export type CreateFAQVariables = {
	title: string;
	content: string;
	isHTML: boolean;
};
export type SuccessfulCreateFAQResponse = {
	success: true;
	faq: Models.FAQ;
};
export type FailedCreateFAQResponse = {
	success: false;
	error: Errors.GenericError;
};
export type CreateFAQResponse = SuccessfulCreateFAQResponse | FailedCreateFAQResponse;
// endregion

// region UpdateFAQ
export type UpdateFAQRequest = {
	token: string;
	id: number;
	title?: string;
	content?: string;
	isHTML?: boolean;
};
export type UpdateFAQData = {
	updateFAQ: {
		id: number;
		title?: string;
		content?: string;
		isHTML?: boolean;

		createdAt?: string;
		createdBy?: {
			id: number;
			name?: string;
		};

		updatedAt?: string;
		updatedBy?: {
			id: number;
			name?: string;
		};
	};
};
export type UpdateFAQVariables = {
	id: number;
	title?: string;
	content?: string;
	isHTML?: boolean;
};
export type SuccessfulUpdateFAQResponse = {
	success: true;
	faq: Models.FAQ;
};
export type FailedUpdateFAQResponse = {
	success: false;
	error: Errors.GenericError;
};
export type UpdateFAQResponse = SuccessfulUpdateFAQResponse | FailedUpdateFAQResponse;
// endregion

// region DeleteFAQ
export type DeleteFAQRequest = {
	token: string;
	id: number;
};
export type DeleteFAQData = {
	deleteFAQ: boolean;
};
export type DeleteFAQVariables = {
	id: number;
};
export type SuccessfulDeleteFAQResponse = {
	success: true;
	deleted: boolean;
};
export type FailedDeleteFAQResponse = {
	success: false;
	error: Errors.GenericError;
};
export type DeleteFAQResponse = SuccessfulDeleteFAQResponse | FailedDeleteFAQResponse;
// endregion

export class FAQ extends Base {
	public static async fetchFAQ(request: FetchFAQRequest): Promise<FetchFAQResponse> {
		const { id } = request;

		const apiResponse = await this.query<FetchFAQData, FetchFAQVariables, FetchFAQRequest>(request, {
			query: gql`
				query faq($id: Int!) {
					faq(id: $id) {
						id
						title
						content
						isHTML

						createdAt
						createdBy {
							id
							name
						}

						updatedAt
						updatedBy {
							id
							name
						}
					}
				}
			`,
			variables: { id }
		});

		if (!apiResponse.success || _.isNil(apiResponse.rawResponse.data) || _.isNil(apiResponse.rawResponse.data.faq))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener la faq.")
			};

		const { data } = apiResponse.rawResponse;

		return {
			success: true,
			faq: Factories.FAQ.fromJSON(data.faq)
		};
	}

	public static async fetchFAQs(request: FetchFAQsRequest = {}): Promise<FetchFAQsResponse> {
		const apiResponse = await this.query<FetchFAQsData, FetchFAQsVariables, FetchFAQsRequest>(request, {
			query: gql`
				{
					faqs {
						id
						title
						content
						isHTML

						createdAt
						createdBy {
							id
							name
						}
					}
				}
			`
		});

		if (
			!apiResponse.success ||
			_.isNil(apiResponse.rawResponse.data) ||
			!_.isArray(apiResponse.rawResponse.data.faqs)
		)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener la lista de faqs.")
			};

		const { data } = apiResponse.rawResponse;

		return {
			success: true,
			faqs: data.faqs.map((faq: any) => Factories.FAQ.fromJSON(faq))
		};
	}

	public static async createFAQ(request: CreateFAQRequest): Promise<CreateFAQResponse> {
		const { token, title, content, isHTML } = request;

		const response = await this.mutate<CreateFAQData, CreateFAQVariables, CreateFAQRequest>(request, {
			mutation: gql`
				mutation createFAQ($title: String!, $content: String!, $isHTML: Boolean!) {
					createFAQ(faq: { title: $title, content: $content, isHTML: $isHTML }) {
						id
						title
						content
						isHTML

						createdAt
						createdBy {
							id
							name
						}

						updatedAt
						updatedBy {
							id
							name
						}
					}
				}
			`,
			variables: { title, content, isHTML },
			context: { token }
		});

		if (!response.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar crear la faq.")
			};

		const { data } = response.rawResponse;

		if (_.isNil(data) || _.isNil(data.createFAQ))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar crear la faq.")
			};

		return {
			success: true,
			faq: Factories.FAQ.fromJSON(data.createFAQ)
		};
	}

	public static async updateFAQ(request: UpdateFAQRequest): Promise<UpdateFAQResponse> {
		const { id, token, title, content, isHTML } = request;

		const response = await this.mutate<UpdateFAQData, UpdateFAQVariables, UpdateFAQRequest>(request, {
			mutation: gql`
				mutation updateFAQ($id: Int!, $title: String, $content: String, $isHTML: Boolean) {
					updateFAQ(id: $id, faq: { title: $title, content: $content, isHTML: $isHTML }) {
						id
						title
						content
						isHTML

						createdAt
						createdBy {
							id
							name
						}

						updatedAt
						updatedBy {
							id
							name
						}
					}
				}
			`,
			variables: { id, title, content, isHTML },
			context: { token }
		});

		if (!response.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar actualizar la faq.")
			};

		const { data } = response.rawResponse;

		if (_.isNil(data) || _.isNil(data.updateFAQ))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar actualizar la faq.")
			};

		return {
			success: true,
			faq: Factories.FAQ.fromJSON(data.updateFAQ)
		};
	}

	public static async deleteFAQ(request: DeleteFAQRequest): Promise<DeleteFAQResponse> {
		const { token, id } = request;

		const apiResponse = await this.mutate<DeleteFAQData, DeleteFAQVariables, DeleteFAQRequest>(request, {
			mutation: gql`
				mutation deleteFAQ($id: Int!) {
					deleteFAQ(id: $id)
				}
			`,
			variables: { id },
			context: { token }
		});

		if (
			!apiResponse.success ||
			_.isNil(apiResponse.rawResponse.data) ||
			_.isNil(apiResponse.rawResponse.data.deleteFAQ)
		)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar eliminar la faq.")
			};

		const { data } = apiResponse.rawResponse;

		return {
			success: true,
			deleted: data.deleteFAQ
		};
	}
}
