import gql from "graphql-tag";
import * as _ from "lodash";
import * as Errors from "../errors";
import * as Factories from "../factories";
import * as Models from "../models";
import { Base } from "./Base";

// region SignIn
export type SignInRequest = {
	email: string;
	password: string;
};
export type SignInData = {
	signIn: {
		token: string;
		user: {
			id: number;
			email?: string;
			name?: string;
			role: string;
			createdAt?: string;
		};
	};
};
export type SignInVariables = {
	email: string;
	password: string;
};
export type SuccessfulSignInResponse = {
	success: true;
	token: string;
	user: Models.User;
};
export type FailedSignInResponse = {
	success: false;
	error: Errors.GenericError;
};
export type SignInResponse = SuccessfulSignInResponse | FailedSignInResponse;
// endregion

// region FetchMe
export type FetchMeRequest = {
	token: string;
};
export type FetchMeData = {
	me: {
		id: number;
		email?: string;
		name?: string;
		role: string;
		createdAt?: string;
	};
};
export type FetchMeVariables = {};
export type SuccessfulFetchMeResponse = {
	success: true;
	user: Models.User;
};
export type FailedFetchMeResponse = {
	success: false;
	error: Errors.GenericError;
};
export type FetchMeResponse = SuccessfulFetchMeResponse | FailedFetchMeResponse;
// endregion

// region SignUp
export type SignUpRequest = {
	email: string;
	password: string;
	name: string;
};
export type SignUpData = {
	signUp: {
		token: string;
		user: {
			id: number;
			email?: string;
			name?: string;
			role: string;
			createdAt?: string;
		};
	};
};
export type SignUpVariables = {
	email: string;
	password: string;
	name: string;
};
export type SuccessfulSignUpResponse = {
	success: true;
	token: string;
	user: Models.User;
};
export type FailedSignUpResponse = {
	success: false;
	error: Errors.GenericError | Errors.ObjectNotFoundError;
};
export type SignUpResponse = SuccessfulSignUpResponse | FailedSignUpResponse;
// endregion

export class User extends Base {
	public static async signIn(request: SignInRequest): Promise<SignInResponse> {
		const { email, password } = request;

		const apiResponse = await this.mutate<SignInData, SignInVariables, SignInRequest>(request, {
			mutation: gql`
				mutation signIn($email: String!, $password: String!) {
					signIn(user: { email: $email, password: $password }) {
						token
						user {
							id
							email
							name
							role
							createdAt
						}
					}
				}
			`,
			variables: { email, password }
		});

		if (!apiResponse.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar loguearse.")
			};

		const { data } = apiResponse.rawResponse;

		if (_.isNil(data) || _.isNil(data.signIn))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar loguearse.")
			};

		const { token, user } = data.signIn;

		if (_.isNil(token) || _.isNil(user))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar loguearse.")
			};

		return {
			success: true,
			token,
			user: Factories.User.fromJSON(user)
		};
	}

	public static async fetchMe(request: FetchMeRequest): Promise<FetchMeResponse> {
		const { token } = request;

		const apiResponse = await this.query<FetchMeData, FetchMeVariables, FetchMeRequest>(request, {
			query: gql`
				{
					me {
						id
						email
						name
						role
						createdAt
					}
				}
			`,
			context: {
				token
			}
		});

		if (!apiResponse.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener información de su usuario.")
			};

		const { data } = apiResponse.rawResponse;

		if (_.isNil(data) || _.isNil(data.me))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar obtener información de su usuario.")
			};

		return {
			success: true,
			user: Factories.User.fromJSON(data.me)
		};
	}

	public static async signUp(request: SignUpRequest): Promise<SignUpResponse> {
		const { email, password, name } = request;

		const response = await this.mutate<SignUpData, SignUpVariables, SignUpRequest>(request, {
			mutation: gql`
				mutation signUp($email: String!, $password: String!, $name: String!) {
					signUp(user: { email: $email, password: $password, name: $name, role: User }) {
						token
						user {
							id
							email
							name
							role
							createdAt
						}
					}
				}
			`,
			variables: { email, password, name }
		});

		if (!response.success)
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar resgistrarlo.")
			};

		const { data } = response.rawResponse;

		if (_.isNil(data) || _.isNil(data.signUp) || _.isNil(data.signUp.user) || _.isNil(data.signUp.token))
			return {
				success: false,
				error: Errors.genericError("Hubo un error al intentar resgistrarlo.")
			};

		const { signUp } = data;

		return {
			success: true,
			user: Factories.User.fromJSON(signUp.user),
			token: signUp.token
		};
	}
}
