import { ReactNativeFile } from "apollo-upload-client";
import bind from "bind-decorator";
import * as _ from "lodash";
import { action, observable, runInAction } from "mobx";
import * as moment from "moment";
import * as API from "src/core/api";
import * as Errors from "../../../errors";
import * as Models from "../../../models";
import { Base } from "./Base";

export type CreateCourseState = {
	code: string;
	name: string;
	eva: string;
	semester: string | number;
	year: string | number;
	icon?: File | ReactNativeFile;
};
export type InitialCreateCourseStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingCreateCourseStatus = {
	isLoading: true;
	success?: undefined;
	progress: number;
};
export type SuccessfulCreateCourseResponse = {
	isLoading: false;
	success: true;
	course: Models.Course;
	progress: 100;
};
export type CreateCourseField = "code" | "name" | "eva" | "semester" | "year" | "icon";
export type FailedCreateCourseResponse = {
	isLoading: false;
	success: false;
	error:
		| Array<Errors.FormValidationError<CreateCourseField>>
		| Errors.GenericError
		| Errors.UserNotAuthenticatedError;
};
export type CreateCourseResponse = SuccessfulCreateCourseResponse | FailedCreateCourseResponse;
export type CreateCourseStatus = InitialCreateCourseStatus | LoadingCreateCourseStatus | CreateCourseResponse;

export class CreateCourse extends Base {
	public initialStatus: CreateCourseStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public initialState: CreateCourseState = {
		code: "",
		name: "",
		eva: "",
		semester: moment().month() >= 5 ? 2 : 1,
		year: moment().year()
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	@action.bound
	public setCode(code: string) {
		this.state.code = code;
	}

	@action.bound
	public setName(name: string) {
		this.state.name = name;
	}

	@action.bound
	public setEva(eva: string) {
		this.state.eva = eva;
	}

	@action.bound
	public setSemester(semester: number | string) {
		this.state.semester = semester;
	}

	@action.bound
	public setYear(year: number | string) {
		this.state.year = year;
	}

	@action.bound
	public setIcon(icon: File | ReactNativeFile | undefined) {
		this.state.icon = icon;
	}

	public validate(): Array<Errors.FormValidationError<CreateCourseField>> {
		const { code, name, semester, year } = this.state;
		const errors: Array<Errors.FormValidationError<CreateCourseField>> = [];

		if (code.trim().length === 0)
			errors.push(
				Errors.formValidationError<CreateCourseField>("code", "EMPTY_FIELD", "Campo code no puede estar vacío.")
			);

		if (name.trim().length === 0)
			errors.push(
				Errors.formValidationError<CreateCourseField>("name", "EMPTY_FIELD", "Campo name no puede estar vacío.")
			);

		if (typeof semester === "string")
			if (semester.trim().length === 0)
				errors.push(
					Errors.formValidationError<CreateCourseField>(
						"semester",
						"EMPTY_FIELD",
						"Campo semester no puede estar vacío."
					)
				);
			else if (Number.isNaN(Number(semester)))
				errors.push(
					Errors.formValidationError<CreateCourseField>(
						"semester",
						"INVALID_VALUE",
						"Campo semestre no es un número válido."
					)
				);

		if (typeof year === "string")
			if (year.trim().length === 0)
				errors.push(
					Errors.formValidationError<CreateCourseField>(
						"year",
						"EMPTY_FIELD",
						"Campo year no puede estar vacío."
					)
				);
			else if (Number.isNaN(Number(year)))
				errors.push(
					Errors.formValidationError<CreateCourseField>(
						"year",
						"INVALID_VALUE",
						"Campo year no es un número válido."
					)
				);

		return errors;
	}

	@bind
	public async create(): Promise<CreateCourseResponse> {
		const { courseStore } = this;
		const { token } = courseStore.store.userStore;
		const { code, name, eva, semester, year, icon } = this.state;

		let newStatus: CreateCourseStatus = {
			isLoading: true,
			success: undefined,
			progress: 0
		};
		this.setStatus(newStatus);

		if (_.isNil(token)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar logueado para crear un curso.")
			};
			return this.setStatus(newStatus);
		}

		const errors = this.validate();

		if (errors.length > 0)
			return this.setStatus({
				isLoading: false,
				success: false,
				error: errors
			});

		const setProgress = (progress: number) => {
			this.setStatus({ isLoading: true, progress });
		};

		const request: API.CreateCourseRequest = {
			token,
			code,
			name,
			eva: _.isNil(eva) || eva.trim().length === 0 ? undefined : eva,
			semester: Number(semester),
			year: Number(year),
			icon,
			onProgress: e => setProgress((e.loaded / e.total) * 100)
		};

		const apiResponse = await API.Course.createCourse(request);

		newStatus = apiResponse.success
			? {
					isLoading: false,
					success: true,
					course: courseStore.saveCourse(apiResponse.course),
					progress: 100
			  }
			: {
					isLoading: false,
					success: false,
					error: apiResponse.error
			  };

		if (newStatus.success) runInAction(() => (this.state = _.cloneDeep(this.initialState)));
		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends CreateCourseStatus>(status: T): T {
		return (this.status = status);
	}
}
