import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import { Base } from "./Base";

export type InitialDeleteCourseStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingDeleteCourseStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulDeleteCourseStatus = {
	isLoading: false;
	success: true;
	deleted: boolean;
};
export type FailedDeleteCourseStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError | Errors.UserNotAuthenticatedError | Errors.NotReadyError;
};

export type DeleteCourseResponse = SuccessfulDeleteCourseStatus | FailedDeleteCourseStatus;
export type DeleteCourseStatus = InitialDeleteCourseStatus | LoadingDeleteCourseStatus | DeleteCourseResponse;

export class DeleteCourse extends Base {
	public initialStatus: DeleteCourseStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	@computed
	public get isReady() {
		return this.code !== undefined;
	}

	@observable
	protected code: string | undefined;

	@action.bound
	public setCode(code: string | undefined) {
		this.code = code;
	}

	public async delete(): Promise<DeleteCourseResponse> {
		const { courseStore, code } = this;
		const { token } = courseStore.store.userStore;

		let newStatus: DeleteCourseStatus = { isLoading: true };
		this.setStatus(newStatus);

		if (_.isNil(token)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar logueado para eliminar el curso.")
			};

			return this.setStatus(newStatus);
		}

		if (_.isNil(code)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.notReadyError("No se está preparado para eliminar el curso.")
			};

			return this.setStatus(newStatus);
		}

		const apiResponse = await API.Course.deleteCourse({ token, courseCode: code });

		if (apiResponse.success) {
			if (apiResponse.deleted) courseStore.deleteCourse(code);

			newStatus = {
				isLoading: false,
				success: true,
				deleted: apiResponse.deleted
			};
		} else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends DeleteCourseStatus>(status: T): T {
		return (this.status = status);
	}
}
