import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import * as Models from "src/core/models";
import { Base } from "./Base";

export type InitialFetchCourseStatus = {
	isLoading: false;
	success?: undefined;
	course?: undefined;
};
export type LoadingFetchCourseStatus = {
	isLoading: true;
	success?: undefined;
	course?: undefined;
};
export type SuccessfulFetchCourseStatus = {
	isLoading: false;
	success: true;
	course: Models.Course;
};
export type FailedFetchCourseStatus = {
	isLoading: false;
	success: false;
	course?: undefined;
	error: Errors.GenericError | Errors.NotReadyError;
};

export type FetchCourseResponse = SuccessfulFetchCourseStatus | FailedFetchCourseStatus;
export type FetchCourseStatus = InitialFetchCourseStatus | LoadingFetchCourseStatus | FetchCourseResponse;

export class FetchCourse extends Base {
	public initialStatus: FetchCourseStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	@observable
	protected course: Models.Course | string | undefined;

	@computed
	public get isReady(): boolean {
		return this.course !== undefined;
	}

	@action.bound
	public setCourse(course: Models.Course | string | undefined) {
		this.course = course;
	}

	public async fetch(): Promise<FetchCourseResponse> {
		const { courseStore, course } = this;
		let newStatus: FetchCourseStatus = { isLoading: true };
		this.setStatus(newStatus);

		if (course === undefined)
			return this.setStatus({
				success: false,
				isLoading: false,
				error: Errors.notReadyError("No se está preparado para obtener el curso.")
			});

		const apiResponse = await API.Course.fetchCourseByCode({
			code: typeof course === "string" ? course : course.code
		});

		newStatus = apiResponse.success
			? {
					isLoading: false,
					success: true,
					course: courseStore.saveCourse(apiResponse.course)
			  }
			: {
					isLoading: false,
					success: false,
					error: apiResponse.error
			  };

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends FetchCourseStatus>(status: T): T {
		return (this.status = status);
	}
}
