import bind from "bind-decorator";
import * as _ from "lodash";
import { action, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "../../../errors";
import * as Models from "../../../models";
import { Base } from "./Base";

export type InitialFetchCoursesStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchCoursesStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchCoursesStatus = {
	isLoading: false;
	success: true;
	courses: Models.Course[];
};
export type FailedFetchCoursesStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError;
};
export type FetchCoursesStatusResponse = SuccessfulFetchCoursesStatus | FailedFetchCoursesStatus;
export type FetchCoursesStatus = InitialFetchCoursesStatus | LoadingFetchCoursesStatus | FetchCoursesStatusResponse;

export class FetchCourses extends Base {
	public initialStatus: FetchCoursesStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	@bind
	public async fetch(): Promise<FetchCoursesStatusResponse> {
		const { courseStore } = this;
		let newStatus: FetchCoursesStatus = {
			isLoading: true,
			success: undefined
		};
		this.setStatus(newStatus);

		const apiResponse = await API.Course.fetchCourses();

		newStatus = apiResponse.success
			? {
					isLoading: false,
					success: true,
					courses: apiResponse.courses.map(course => courseStore.saveCourse(course))
			  }
			: {
					isLoading: false,
					success: false,
					error: apiResponse.error
			  };

		return Promise.resolve(this.setStatus(newStatus));
	}

	@action
	protected setStatus<T extends FetchCoursesStatus>(status: T): T {
		return (this.status = status);
	}
}
