import { ReactNativeFile } from "apollo-upload-client";
import bind from "bind-decorator";
import * as _ from "lodash";
import { action, computed, observable, runInAction } from "mobx";
import * as moment from "moment";
import * as API from "src/core/api";
import * as Errors from "../../../errors";
import { Helper } from "../../../Helper";
import * as Models from "../../../models";
import { Base } from "./Base";

export type UpdateCourseState = {
	code: string;
	name: string;
	eva: string;
	semester: string | number;
	year: string | number;
	icon?: File | ReactNativeFile;
};

export type InitialUpdateCourseStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingUpdateCourseStatus = {
	isLoading: true;
	success?: undefined;
	progress: number;
};
export type SuccessfulUpdateCourseStatus = {
	isLoading: false;
	success: true;
	course: Models.Course;
	progress: 100;
};
export type UpdateCourseField = "code" | "name" | "eva" | "semester" | "year" | "icon";
export type FailedUpdateCourseStatus = {
	isLoading: false;
	success: false;
	errors:
		| Errors.GenericError
		| Errors.NotReadyError
		| Errors.UserNotAuthenticatedError
		| Array<Errors.FormValidationError<UpdateCourseField>>;
};
export type UpdateCourseStatusResponse = SuccessfulUpdateCourseStatus | FailedUpdateCourseStatus;
export type UpdateCourseStatus = InitialUpdateCourseStatus | LoadingUpdateCourseStatus | UpdateCourseStatusResponse;

export class UpdateCourse extends Base {
	public initialStatus: UpdateCourseStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public initialState: UpdateCourseState = {
		code: "",
		name: "",
		eva: "",
		semester: moment().month() >= 5 ? 2 : 1,
		year: moment().year()
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	protected course: Models.Course | undefined;

	@computed
	public get isReady(): boolean {
		return this.course !== undefined;
	}

	@action.bound
	public setCode(code: string) {
		this.state.code = code;
	}

	@action.bound
	public setName(name: string) {
		this.state.name = name;
	}

	@action.bound
	public setEva(eva: string) {
		this.state.eva = eva;
	}

	@action.bound
	public setSemester(semester: number | string) {
		this.state.semester = semester;
	}

	@action.bound
	public setYear(year: number | string) {
		this.state.year = year;
	}

	@action.bound
	public setIcon(icon: File | ReactNativeFile | undefined) {
		this.state.icon = icon;
	}

	@action
	public setCourse(course: Models.Course | undefined, updateState: boolean = true) {
		this.course = course;

		if (updateState)
			this.state = course
				? {
						code: Helper.trimOrEmptyString(course.code),
						eva: Helper.trimOrEmptyString(course.eva),
						name: Helper.trimOrEmptyString(course.name),
						semester: course.semester!.toString(),
						year: course.year!.toString()
				  }
				: _.cloneDeep(this.initialState);
	}

	public validate(): Array<Errors.FormValidationError<UpdateCourseField>> {
		const { code, name, semester, year } = this.state;
		const errors: Array<Errors.FormValidationError<UpdateCourseField>> = [];

		if (code.trim().length === 0)
			errors.push(
				Errors.formValidationError<UpdateCourseField>("code", "EMPTY_FIELD", "Campo code no puede estar vacío.")
			);

		if (name.trim().length === 0)
			errors.push(
				Errors.formValidationError<UpdateCourseField>("name", "EMPTY_FIELD", "Campo name no puede estar vacío.")
			);

		if (typeof semester === "string")
			if (semester.trim().length === 0)
				errors.push(
					Errors.formValidationError<UpdateCourseField>(
						"semester",
						"EMPTY_FIELD",
						"Campo semester no puede estar vacío."
					)
				);
			else if (Number.isNaN(Number(semester)))
				errors.push(
					Errors.formValidationError<UpdateCourseField>(
						"semester",
						"INVALID_VALUE",
						"Campo semestre no es un número válido."
					)
				);

		if (typeof year === "string")
			if (year.trim().length === 0)
				errors.push(
					Errors.formValidationError<UpdateCourseField>(
						"year",
						"EMPTY_FIELD",
						"Campo year no puede estar vacío."
					)
				);
			else if (Number.isNaN(Number(year)))
				errors.push(
					Errors.formValidationError<UpdateCourseField>(
						"year",
						"INVALID_VALUE",
						"Campo year no es un número válido."
					)
				);

		return errors;
	}

	@bind
	public async update(): Promise<UpdateCourseStatusResponse> {
		const { courseStore, course } = this;
		const { token } = this.courseStore.store.userStore;
		const { code, name, eva, semester, year, icon } = this.state;

		let newStatus: UpdateCourseStatus = {
			isLoading: true,
			success: undefined,
			progress: 0
		};
		this.setStatus(newStatus);

		if (_.isNil(token)) {
			newStatus = {
				isLoading: false,
				success: false,
				errors: Errors.userNotAuthenticatedError("Debe estar logueado para actualizar un curso.")
			};

			return Promise.resolve(this.setStatus(newStatus));
		}

		if (_.isNil(course)) {
			newStatus = {
				isLoading: false,
				success: false,
				errors: Errors.notReadyError("No se está preparado para actualizar el curso.")
			};

			return Promise.resolve(this.setStatus(newStatus));
		}

		const errors = this.validate();

		if (errors.length > 0)
			return Promise.resolve(
				this.setStatus({
					isLoading: false,
					success: false,
					errors
				})
			);

		const setProgress = (progress: number) => {
			this.setStatus({ isLoading: true, progress });
		};

		const request: API.UpdateCourseRequest = {
			id: course.id,
			token,
			code,
			name,
			eva: _.isNil(eva) || eva.trim().length === 0 ? undefined : eva,
			semester: Number(semester),
			year: Number(year),
			icon,
			onProgress: e => setProgress((e.loaded / e.total) * 100)
		};

		const apiResponse = await API.Course.updateCourse(request);

		newStatus = apiResponse.success
			? {
					isLoading: false,
					success: true,
					course: courseStore.saveCourse(apiResponse.course),
					progress: 100
			  }
			: {
					isLoading: false,
					success: false,
					errors: apiResponse.error
			  };

		if (newStatus.success) runInAction(() => (this.state = _.cloneDeep(this.initialState)));
		return Promise.resolve(this.setStatus(newStatus));
	}

	@action
	protected setStatus<T extends UpdateCourseStatus>(status: T): T {
		return (this.status = status);
	}
}
