import bind from "bind-decorator";
import * as _ from "lodash";
import { action, autorun, computed, observable, runInAction } from "mobx";
import * as API from "src/core/api";
import * as Errors from "../../../errors";
import * as Models from "../../../models";
import { Base } from "./Base";

export type CreateCourseClassState = {
	number: number | string;
	title: string;
	videoURL: string;
	disabled: boolean;
};

export type InitialCreateCourseClassStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingCreateCourseClassStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulCreateCourseClassStatus = {
	isLoading: false;
	success: true;
	courseClass: Models.CourseClass;
};
export type CreateCourseClassField = "number" | "title" | "videoURL" | "disabled";
export type FailedCreateCourseClassStatus = {
	isLoading: false;
	success: false;
	error:
		| Array<Errors.FormValidationError<CreateCourseClassField>>
		| Errors.GenericError
		| Errors.UserNotAuthenticatedError
		| Errors.ObjectNotFoundError
		| Errors.NotReadyError;
};
export type CreateCourseClassStatusResponse = SuccessfulCreateCourseClassStatus | FailedCreateCourseClassStatus;
export type CreateCourseClassStatus =
	| InitialCreateCourseClassStatus
	| LoadingCreateCourseClassStatus
	| CreateCourseClassStatusResponse;

export class CreateCourseClass extends Base {
	public initialStatus: CreateCourseClassStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public initialState: CreateCourseClassState = {
		number: 1,
		title: "",
		videoURL: "",
		disabled: false
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	protected course: Models.Course | undefined;
	protected videoURLAutoGenerator: () => void;

	protected onCreated() {
		super.onCreated();

		_.defer(() => {
			this.videoURLAutoGenerator = autorun(() => {
				const { state, course } = this;
				const { number } = state;

				let classNumber: string;
				let courseCode: string;

				if (typeof number === "number")
					classNumber = Math.floor(number / 10) === 0 ? `0${number.toString()}` : number.toString();
				else {
					classNumber = number.length > 0 ? number : "x";

					while (classNumber.length < 2){
						classNumber = `0${classNumber}`;	
					} 
				}

				courseCode = course ? course.code : "xx";

				runInAction(
					() =>
						(state.videoURL = `http://openfing-video.fing.edu.uy/media/${courseCode}/${courseCode}_${classNumber}.webm`)
				);
			});
		});
	}

	@computed
	public get isReady(): boolean {
		return this.course !== undefined;
	}

	@action.bound
	public setNumber(number: string | number) {
		this.state.number = number;
	}

	@action.bound
	public setTitle(title: string) {
		this.state.title = title;
	}

	@action.bound
	public setVideoURL(videoURL: string) {
		this.videoURLAutoGenerator();
		this.state.videoURL = videoURL;
	}

	@action.bound
	public setDisabled(disabled: boolean) {
		this.state.disabled = disabled;
	}

	@action
	public setCourse(course: Models.Course) {
		const { state } = this;

		this.course = course;

		const { classes } = course;

		let number = state.number;

		if (classes !== undefined)
			classes.forEach(courseClass => {
				if (courseClass.number !== undefined && courseClass.number >= number) number = courseClass.number + 1;
			});

		state.number = number;
	}

	public processInfo():
		| Array<Errors.FormValidationError<CreateCourseClassField>>
		| {
				number: number;
				title: string;
				videoURL?: string;
				disabled?: boolean;
		  } {
		const { state } = this;
		const errors: Array<Errors.FormValidationError<CreateCourseClassField>> = [];

		let number: number;
		if (typeof state.number !== "number")
			if (state.number.trim().length === 0) {
				errors.push(
					Errors.formValidationError<CreateCourseClassField>(
						"number",
						"EMPTY_FIELD",
						"Campo numero no puede estar vacío."
					)
				);
				number = 0;
			} else {
				number = Number(state.number);

				if (Number.isNaN(number))
					errors.push(
						Errors.formValidationError<CreateCourseClassField>(
							"number",
							"INVALID_VALUE",
							"El campo number no tiene un valor válido."
						)
					);
			}
		else number = state.number;

		const title = state.title.trim();
		if (state.title.trim().length === 0)
			errors.push(
				Errors.formValidationError<CreateCourseClassField>(
					"title",
					"EMPTY_FIELD",
					"Campo title no puede estar vacío."
				)
			);

		let { videoURL } = state;
		if (videoURL !== undefined) videoURL = videoURL.trim();

		const { disabled } = state;

		if (errors.length > 0) return errors;

		return {
			number,
			title,
			videoURL,
			disabled
		};
	}

	@bind
	public async create(): Promise<CreateCourseClassStatusResponse> {
		const { courseClassStore, course } = this;
		const { userStore } = courseClassStore.store;
		const { token } = userStore;

		this.setStatus({
			isLoading: true,
			success: undefined
		});

		if (_.isNil(token))
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar logueado para crear un curso.")
			});

		const courseId = course === undefined ? undefined : course.id;
		if (courseId === undefined)
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.notReadyError("No se está preparado para crear la clase.")
			});

		const processedInfo = this.processInfo();

		if (Array.isArray(processedInfo))
			return this.setStatus({
				isLoading: false,
				success: false,
				error: processedInfo
			});

		const apiResponse = await API.CourseClass.createCourseClass({
			...processedInfo,
			token,
			courseId
		});

		let newStatus: CreateCourseClassStatusResponse;

		if (apiResponse.success) {
			const courseClass = courseClassStore.saveCourseClass(apiResponse.courseClass);

			runInAction(() => (courseClass.course = course));

			newStatus = {
				isLoading: false,
				success: true,
				courseClass
			};
		} else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends CreateCourseClassStatus>(status: T): T {
		return (this.status = status);
	}
}
