import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import * as Models from "src/core/models";
import { Base } from "./Base";

export type InitialDeleteCourseClassStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingDeleteCourseClassStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulDeleteCourseClassStatus = {
	isLoading: false;
	success: true;
	deleted: boolean;
};
export type FailedDeleteCourseClassStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError | Errors.UserNotAuthenticatedError | Errors.NotReadyError;
};

export type DeleteCourseClassResponse = SuccessfulDeleteCourseClassStatus | FailedDeleteCourseClassStatus;
export type DeleteCourseClassStatus =
	| InitialDeleteCourseClassStatus
	| LoadingDeleteCourseClassStatus
	| DeleteCourseClassResponse;

export class DeleteCourseClass extends Base {
	public initialStatus: DeleteCourseClassStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	@observable
	protected courseClass: Models.CourseClass | number | undefined;

	@computed
	public get isReady(): boolean {
		return this.courseClass !== undefined;
	}

	@action.bound
	public setCourseClass(courseClass: Models.CourseClass | number | undefined) {
		this.courseClass = courseClass;
	}

	public async delete(): Promise<DeleteCourseClassResponse> {
		const { courseClassStore, courseClass } = this;
		const { token } = this.courseClassStore.store.userStore;

		let newStatus: DeleteCourseClassStatus = { isLoading: true };
		this.setStatus(newStatus);

		if (_.isNil(token)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar logueado para eliminar la clase.")
			};

			return this.setStatus(newStatus);
		}

		if (_.isNil(courseClass)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.notReadyError("No se está preparado para eliminar la clase.")
			};

			return this.setStatus(newStatus);
		}

		const classId = typeof courseClass === "number" ? courseClass : courseClass.id;

		if (Number.isNaN(classId)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.genericError("La clase ingresada no es válida.")
			};

			this.setStatus(newStatus);
			return Promise.resolve(newStatus);
		}

		const apiResponse = await API.CourseClass.deleteCourseClass({
			token,
			id: classId
		});

		if (apiResponse.success) {
			if (apiResponse.deleted) courseClassStore.deleteCourseClass(classId);

			newStatus = {
				isLoading: false,
				success: true,
				deleted: apiResponse.deleted
			};
		} else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		this.setStatus(newStatus);
		return newStatus;
	}

	@action
	protected setStatus<T extends DeleteCourseClassStatus>(status: T): T {
		return (this.status = status);
	}
}
