import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import * as Models from "src/core/models";
import { Base } from "./Base";

export type InitialFetchCourseClassStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchCourseClassStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchCourseClassStatus = {
	isLoading: false;
	success: true;
	courseClass: Models.CourseClass;
};
export type FailedFetchCourseClassStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError | Errors.NotReadyError;
};
export type FetchCourseClassResponse = SuccessfulFetchCourseClassStatus | FailedFetchCourseClassStatus;
export type FetchCourseClassStatus =
	| InitialFetchCourseClassStatus
	| LoadingFetchCourseClassStatus
	| FetchCourseClassResponse;

export class FetchCourseClass extends Base {
	public initialStatus: FetchCourseClassStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	@observable
	protected courseClass:
		| {
				courseCode: string;
				classNo: number | string;
		  }
		| undefined;

	@computed
	public get isReady(): boolean {
		return this.courseClass !== undefined;
	}

	@action.bound
	public setData(
		courseClass:
			| {
					courseCode: string;
					classNo: number | string;
			  }
			| undefined
	) {
		this.courseClass = courseClass;
	}

	public async fetch(): Promise<FetchCourseClassResponse> {
		const { courseClassStore, courseClass } = this;

		let newStatus: FetchCourseClassStatus = { isLoading: true };
		this.setStatus(newStatus);

		if (courseClass === undefined)
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.notReadyError("No se está preparado para obtener la clase.")
			});

		const { courseCode } = courseClass;
		const classNo = typeof courseClass.classNo === "number" ? courseClass.classNo : Number(courseClass.classNo);

		if (Number.isNaN(classNo))
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.genericError("El número de clase ingresado no es correcto.")
			});

		const apiResponse = await API.CourseClass.fetchCourseClass({
			courseCode,
			classNo
		});

		if (apiResponse.success)
			newStatus = {
				isLoading: false,
				success: true,
				courseClass: courseClassStore.saveCourseClass(apiResponse.courseClass)
			};
		else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends FetchCourseClassStatus>(status: T): T {
		return (this.status = status);
	}
}
