import bind from "bind-decorator";
import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import * as API from "src/core/api";
import { Helper } from "src/core/Helper";
import * as Errors from "../../../errors";
import * as Models from "../../../models";
import { Base } from "./Base";

// region typedefs
export type UpdateCourseClassState = {
	id: string | number | undefined;
	number?: number | string;
	title?: string;
	videoURL?: string;
	disabled?: boolean;
};

export type InitialUpdateCourseClassStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingUpdateCourseClassStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulUpdateCourseClassStatus = {
	isLoading: false;
	success: true;
	courseClass: Models.CourseClass;
};
export type UpdateCourseClassField = "id" | "number" | "title" | "videoURL" | "disabled";
export type FailedUpdateCourseClassStatus = {
	isLoading: false;
	success: false;
	error:
		| Array<Errors.FormValidationError<UpdateCourseClassField>>
		| Errors.GenericError
		| Errors.UserNotAuthenticatedError;
};
export type UpdateCourseClassStatusResponse = SuccessfulUpdateCourseClassStatus | FailedUpdateCourseClassStatus;
export type UpdateCourseClassStatus =
	| InitialUpdateCourseClassStatus
	| LoadingUpdateCourseClassStatus
	| UpdateCourseClassStatusResponse;
// endregion

export class UpdateCourseClass extends Base {
	public initialStatus: UpdateCourseClassStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public initialState: UpdateCourseClassState = {
		id: "",
		number: "",
		title: "",
		videoURL: "",
		disabled: false
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	@observable
	protected courseClass: Models.CourseClass | undefined;

	@computed
	public get isReady(): boolean {
		return this.courseClass !== undefined;
	}

	@action.bound
	public setCourse(courseClass: Models.CourseClass | undefined, updateState: boolean = true) {
		this.courseClass = courseClass;

		if (updateState)
			this.state = {
				id: (courseClass && courseClass.id) || "",
				number: (courseClass && courseClass.number) || "",
				title: (courseClass && courseClass.title) || "",
				videoURL: (courseClass && courseClass.videoURL) || "",
				disabled: (courseClass && courseClass.disabled) || false
			};
	}

	// region setters
	@action.bound
	public setId(id: string) {
		this.state.id = id;
	}

	@action.bound
	public setNumber(number: number | string) {
		this.state.number = number;
	}

	@action.bound
	public setTitle(title: string) {
		this.state.title = title;
	}

	@action.bound
	public setVideoURL(videoURL: string) {
		this.state.videoURL = videoURL;
	}

	@action.bound
	public setDisabled(disabled: boolean) {
		this.state.disabled = disabled;
	}
	// endregion

	public processInfo():
		| Array<Errors.FormValidationError<UpdateCourseClassField>>
		| {
				id: number;
				number?: number;
				title?: string;
				videoURL?: string;
				disabled?: boolean;
		  } {
		const { state, courseClass } = this;
		const errors: Array<Errors.FormValidationError<UpdateCourseClassField>> = [];

		let id: number;
		if (typeof state.id === "undefined")
			errors.push(
				Errors.formValidationError<UpdateCourseClassField>(
					"id",
					"EMPTY_FIELD",
					"El campo id no puede quedar vacío."
				)
			);
		else if (typeof state.id === "string") {
			id = Number(state.id);

			if (Number.isNaN(id))
				errors.push(
					Errors.formValidationError<UpdateCourseClassField>(
						"id",
						"INVALID_VALUE",
						"El campo id tiene un valor inválido."
					)
				);
		} else id = state.id;

		let number: string | number =
			state.number === undefined ? "" : typeof state.number === "string" ? state.number.trim() : state.number;
		if (typeof number === "string")
			if (number.length !== 0) {
				number = Number(number);

				if (Number.isNaN(number))
					errors.push(
						Errors.formValidationError<UpdateCourseClassField>(
							"number",
							"INVALID_VALUE",
							"El campo number no tiene un valor válido."
						)
					);
			}

		const title = state.title === undefined ? "" : state.title.trim();
		const videoURL = state.videoURL === undefined ? "" : state.videoURL.trim();
		const disabled = state.disabled !== undefined && state.disabled;

		if (errors.length > 0) return errors;

		const result: {
			id: number;
			number?: number;
			title?: string;
			videoURL?: string;
			disabled?: boolean;
		} = {
			id: id!,
			number: typeof number === "string" ? undefined : number,
			title,
			videoURL,
			disabled
		};

		if (!_.isNil(courseClass)) {
			if (Helper.toStringToCompare(courseClass.number) === Helper.toStringToCompare(number))
				result.number = undefined;

			if (Helper.toStringToCompare(courseClass.title) === Helper.toStringToCompare(title))
				result.title = undefined;

			if (Helper.toStringToCompare(courseClass.videoURL) === Helper.toStringToCompare(videoURL))
				result.videoURL = undefined;

			if (Helper.toStringToCompare(courseClass.disabled) === Helper.toStringToCompare(disabled))
				result.disabled = undefined;
		}

		return result;
	}

	@bind
	public async update(): Promise<UpdateCourseClassStatusResponse> {
		const { courseClassStore } = this;
		const { token } = courseClassStore.store.userStore;

		let newStatus: UpdateCourseClassStatus = {
			isLoading: true,
			success: undefined
		};
		this.setStatus(newStatus);

		if (_.isNil(token)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar logueado para crear un curso.")
			};
			return Promise.resolve(this.setStatus(newStatus));
		}

		const processedInfo = this.processInfo();

		if (Array.isArray(processedInfo))
			return Promise.resolve(
				this.setStatus({
					isLoading: false,
					success: false,
					error: processedInfo
				})
			);

		const request: API.UpdateCourseClassRequest = {
			...processedInfo,
			token
		};

		const apiResponse = await API.CourseClass.updateCourseClass(request);

		newStatus = apiResponse.success
			? {
					isLoading: false,
					success: true,
					courseClass: courseClassStore.saveCourseClass(apiResponse.courseClass)
			  }
			: {
					isLoading: false,
					success: false,
					error: apiResponse.error
			  };

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends UpdateCourseClassStatus>(status: T): T {
		return (this.status = status);
	}
}
