import bind from "bind-decorator";
import * as _ from "lodash";
import { action, observable } from "mobx";
import * as API from "src/core/api";
import { UpdateFAQField } from "src/core/store/faq/actions/UpdateFAQ";
import * as Errors from "../../../errors";
import * as Models from "../../../models";
import { Base } from "./Base";

export type CreateFAQState = {
	title: string;
	content: string;
	isHTML: boolean;
};

export type InitialCreateFAQStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingCreateFAQStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulCreateFAQStatus = {
	isLoading: false;
	success: true;
	faq: Models.FAQ;
};
export type CreateFAQField = "title" | "content" | "isHTML";
export type FailedCreateFAQStatus = {
	isLoading: false;
	success: false;
	error: Array<Errors.FormValidationError<CreateFAQField>> | Errors.GenericError | Errors.UserNotAuthenticatedError;
};
export type CreateFAQStatusResponse = SuccessfulCreateFAQStatus | FailedCreateFAQStatus;
export type CreateFAQStatus = InitialCreateFAQStatus | LoadingCreateFAQStatus | CreateFAQStatusResponse;

export class CreateFAQ extends Base {
	public initialStatus: CreateFAQStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public initialState: CreateFAQState = {
		title: "",
		content: "",
		isHTML: false
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	// region setters
	@action.bound
	public setTitle(title: string) {
		this.state.title = title;
	}

	@action.bound
	public setContent(content: string) {
		this.state.content = content;
	}

	@action.bound
	public setIsHTML(isHTML: boolean) {
		this.state.isHTML = isHTML;
	}
	// endregion

	public processInfo():
		| Array<Errors.FormValidationError<CreateFAQField>>
		| {
				title: string;
				content: string;
				isHTML: boolean;
		  } {
		const { state } = this;
		const errors: Array<Errors.FormValidationError<UpdateFAQField>> = [];

		const title = state.title.trim(); // TODO: validate
		if (title.length === 0)
			errors.push(
				Errors.formValidationError<CreateFAQField>(
					"title",
					"EMPTY_FIELD",
					"El campo title no puede estar vacío."
				)
			);

		const content = state.content.trim();
		if (content.length === 0)
			errors.push(
				Errors.formValidationError<CreateFAQField>(
					"content",
					"EMPTY_FIELD",
					"El campo content no puede estar vacío."
				)
			);

		const isHTML = state.isHTML;

		if (errors.length > 0) return errors;

		return {
			title,
			content,
			isHTML
		};
	}

	@bind
	public async create(): Promise<CreateFAQStatusResponse> {
		const { faqStore } = this;
		const { token } = faqStore.store.userStore;

		let newStatus: CreateFAQStatus = {
			isLoading: true,
			success: undefined
		};
		this.setStatus(newStatus);

		if (_.isNil(token)) {
			newStatus = {
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar logueado para crear una faq.")
			};

			return Promise.resolve(this.setStatus(newStatus));
		}

		const processedInfo = this.processInfo();

		if (Array.isArray(processedInfo))
			return Promise.resolve(
				this.setStatus({
					isLoading: false,
					success: false,
					error: processedInfo
				})
			);

		const request: API.CreateFAQRequest = {
			...processedInfo,
			token
		};

		const apiResponse = await API.FAQ.createFAQ(request);

		newStatus = apiResponse.success
			? {
					isLoading: false,
					success: true,
					faq: faqStore.saveFAQ(apiResponse.faq)
			  }
			: {
					isLoading: false,
					success: false,
					error: apiResponse.error
			  };

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends CreateFAQStatus>(status: T): T {
		return (this.status = status);
	}
}
