import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import { Models } from "src/core";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import { Base } from "./Base";

export type InitialDeleteFAQStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingDeleteFAQStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulDeleteFAQStatus = {
	isLoading: false;
	success: true;
	deleted: boolean;
};
export type FailedDeleteFAQStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError | Errors.UserNotAuthenticatedError | Errors.NotReadyError;
};

export type DeleteFAQResponse = SuccessfulDeleteFAQStatus | FailedDeleteFAQStatus;
export type DeleteFAQStatus = InitialDeleteFAQStatus | LoadingDeleteFAQStatus | DeleteFAQResponse;

export class DeleteFAQ extends Base {
	public initialStatus: DeleteFAQStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	@observable
	protected faq: Models.FAQ | string | number;

	@computed
	public get isReady(): boolean {
		return this.faq !== undefined;
	}

	@action.bound
	public setFAQ(faq: Models.FAQ | string | number) {
		this.faq = faq;
	}

	public async delete(): Promise<DeleteFAQResponse> {
		const { faq, faqStore } = this;
		const { token } = faqStore.store.userStore;

		let newStatus: DeleteFAQStatus = { isLoading: true };
		this.setStatus(newStatus);

		if (_.isNil(token))
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar logueado para eliminar una faq.")
			});

		if (_.isNil(faq))
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.notReadyError("No se está preparado para eliminar la faq.")
			});

		const id: number = typeof faq === "object" ? faq.id : typeof faq === "string" ? Number(faq) : faq;

		if (Number.isNaN(id))
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.genericError("El id especificado no es válido.")
			});

		const apiResponse = await API.FAQ.deleteFAQ({ token, id });

		if (apiResponse.success) {
			faqStore.deleteFAQ(id);

			newStatus = {
				isLoading: false,
				success: true,
				deleted: apiResponse.deleted
			};
		} else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends DeleteFAQStatus>(status: T): T {
		return (this.status = status);
	}
}
