import * as _ from "lodash";
import { action, computed, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import * as Models from "src/core/models";
import { Base } from "./Base";

export type FetchFAQRequest = {
	id: number | string;
};
export type InitialFetchFAQStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchFAQStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchFAQStatus = {
	isLoading: false;
	success: true;
	faq: Models.FAQ;
};
export type FailedFetchFAQStatus = {
	isLoading: false;
	success: false;
	error: Errors.GenericError | Errors.NotReadyError;
};

export type FetchFAQResponse = SuccessfulFetchFAQStatus | FailedFetchFAQStatus;
export type FetchFAQStatus = InitialFetchFAQStatus | LoadingFetchFAQStatus | FetchFAQResponse;

export class FetchFAQ extends Base {
	public initialStatus: FetchFAQStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	@observable
	protected faq: Models.FAQ | number | string | undefined;

	@computed
	public get isReady(): boolean {
		return this.faq !== undefined;
	}

	@action.bound
	public setFAQ(faq: string | number | Models.FAQ) {
		this.faq = faq;
	}

	public async fetch(): Promise<FetchFAQResponse> {
		const { faqStore, faq } = this;
		let newStatus: FetchFAQStatus = { isLoading: true };
		this.setStatus(newStatus);

		if (faq === undefined)
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.notReadyError("No se está preparado para obtener la faq.")
			});

		const id = typeof faq === "number" ? faq : typeof faq === "string" ? Number(faq) : faq.id;

		if (Number.isNaN(id))
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.genericError("El id especificado no es válido.")
			});

		const apiResponse = await API.FAQ.fetchFAQ({ id });

		if (apiResponse.success)
			newStatus = {
				isLoading: false,
				success: true,
				faq: faqStore.saveFAQ(apiResponse.faq)
			};
		else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		this.setStatus(newStatus);

		return newStatus;
	}

	@action
	protected setStatus<T extends FetchFAQStatus>(status: T): T {
		return (this.status = status);
	}
}
