import bind from "bind-decorator";
import * as _ from "lodash";
import { action, computed, observable, runInAction } from "mobx";
import * as API from "src/core/api";
import { Helper } from "src/core/Helper";
import * as Errors from "../../../errors";
import * as Models from "../../../models";
import { Base } from "./Base";

// region typedefs
export type UpdateFAQState = {
	title: string;
	content: string;
	isHTML: boolean;
};

export type InitialUpdateFAQStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingUpdateFAQStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulUpdateFAQStatus = {
	isLoading: false;
	success: true;
	faq: Models.FAQ;
};
export type UpdateFAQField = "title" | "content" | "isHTML";
export type FailedUpdateFAQStatus = {
	isLoading: false;
	success: false;
	errors:
		| Array<Errors.FormValidationError<UpdateFAQField>>
		| Errors.GenericError
		| Errors.UserNotAuthenticatedError
		| Errors.NotReadyError;
};
export type UpdateFAQStatusResponse = SuccessfulUpdateFAQStatus | FailedUpdateFAQStatus;
export type UpdateFAQStatus = InitialUpdateFAQStatus | LoadingUpdateFAQStatus | UpdateFAQStatusResponse;
// endregion

export class UpdateFAQ extends Base {
	public initialStatus: UpdateFAQStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public initialState: UpdateFAQState = {
		title: "",
		content: "",
		isHTML: false
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	@observable
	protected faq: Models.FAQ | undefined;

	@computed
	public get isReady(): boolean {
		return this.faq !== undefined;
	}

	// region setters
	@action.bound
	public setTitle(title: string) {
		this.state.title = title;
	}

	@action.bound
	public setContent(content: string) {
		this.state.content = content;
	}

	@action.bound
	public setIsHTML(isHTML: boolean) {
		this.state.isHTML = isHTML;
	}

	@action
	public setFAQ(faq: Models.FAQ | undefined, updateState: boolean = true) {
		this.faq = faq;

		if (updateState)
			this.state = faq
				? {
						title: faq.title || "",
						content: faq.content || "",
						isHTML: faq.isHTML || false
				  }
				: _.cloneDeep(this.initialState);
	}
	// endregion

	public processInfo():
		| Array<Errors.FormValidationError<UpdateFAQField>>
		| {
				title?: string;
				content?: string;
				isHTML?: boolean;
		  } {
		const { state, faq } = this;
		const errors: Array<Errors.FormValidationError<UpdateFAQField>> = [];

		const title = state.title === undefined ? "" : state.title.trim(); // TODO: validate
		const content = state.content === undefined ? "" : state.content.trim();
		const isHTML = state.isHTML !== undefined && state.isHTML;

		if (errors.length > 0) return errors;

		const result: {
			title?: string;
			content?: string;
			isHTML?: boolean;
		} = {
			title,
			content,
			isHTML
		};

		if (!_.isNil(faq)) {
			if (Helper.toStringToCompare(faq.title) === Helper.toStringToCompare(title)) result.title = undefined;

			if (Helper.toStringToCompare(faq.content) === Helper.toStringToCompare(content)) result.content = undefined;

			if (Helper.toStringToCompare(faq.isHTML) === Helper.toStringToCompare(isHTML)) result.isHTML = undefined;
		}

		return result;
	}

	@bind
	public async update(): Promise<UpdateFAQStatusResponse> {
		const { faqStore, faq } = this;
		const { token } = faqStore.store.userStore;

		let newStatus: UpdateFAQStatus = {
			isLoading: true,
			success: undefined
		};
		this.setStatus(newStatus);

		if (_.isNil(token))
			return this.setStatus({
				isLoading: false,
				success: false,
				errors: Errors.userNotAuthenticatedError("Debe estar logueado para actualizar el faq.")
			});

		if (_.isNil(faq))
			return this.setStatus({
				isLoading: false,
				success: false,
				errors: Errors.notReadyError("No se está preparado para actualizar la faq.")
			});

		const { id } = faq;
		const processedInfo = this.processInfo();

		if (Array.isArray(processedInfo))
			return Promise.resolve(
				this.setStatus({
					isLoading: false,
					success: false,
					errors: processedInfo
				})
			);

		const request: API.UpdateFAQRequest = {
			...processedInfo,
			id,
			token
		};

		const apiResponse = await API.FAQ.updateFAQ(request);

		newStatus = apiResponse.success
			? {
					isLoading: false,
					success: true,
					faq: faqStore.saveFAQ(apiResponse.faq)
			  }
			: {
					isLoading: false,
					success: false,
					errors: apiResponse.error
			  };

		if (newStatus.success) runInAction(() => (this.state = _.cloneDeep(this.initialState)));
		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends UpdateFAQStatus>(status: T): T {
		return (this.status = status);
	}
}
