import * as _ from "lodash";
import { action, observable } from "mobx";
import * as API from "src/core/api";
import * as Errors from "src/core/errors";
import * as Models from "src/core/models";
import { Base } from "./Base";

export type InitialFetchMeStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingFetchMeStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulFetchMeResponse = {
	isLoading: false;
	success: true;
	user: Models.User;
};
export type FailedFetchMeResponse = {
	isLoading: false;
	success: false;
	error: Errors.GenericError | Errors.UserNotAuthenticatedError;
};
export type FetchMeResponse = SuccessfulFetchMeResponse | FailedFetchMeResponse;
export type FetchMeStatus = InitialFetchMeStatus | LoadingFetchMeStatus | FetchMeResponse;

export class FetchMe extends Base {
	public initialStatus: FetchMeStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public async fetch(): Promise<FetchMeStatus> {
		const { userStore } = this;
		const { token } = userStore;

		let newStatus: FetchMeStatus = {
			isLoading: true
		};
		this.setStatus(newStatus);

		if (token === undefined)
			return this.setStatus({
				isLoading: false,
				success: false,
				error: Errors.userNotAuthenticatedError("Debe estar autenticado.")
			});

		const apiResponse = await API.User.fetchMe({ token });

		if (apiResponse.success) {
			const user = userStore.saveUser(apiResponse.user);
			userStore.setCurrentUser(user);

			newStatus = {
				isLoading: false,
				success: true,
				user
			};
		} else
			newStatus = {
				isLoading: false,
				success: false,
				error: apiResponse.error
			};

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends FetchMeStatus>(newStatus: T): T {
		return (this.status = newStatus);
	}
}
