import bind from "bind-decorator";
import * as _ from "lodash";
import { action, observable } from "mobx";
import * as API from "src/core/api";
import * as Models from "src/core/models";
import * as Errors from "../../../errors";
import { Base } from "./Base";

export type SignInState = {
	email: string;
	password: string;
};

export type InitialSignInStatus = {
	isLoading: false;
	success?: undefined;
};
export type LoadingSignInStatus = {
	isLoading: true;
	success?: undefined;
};
export type SuccessfulSignInStatus = {
	isLoading: false;
	success: true;
	user: Models.User;
	token: string;
};
export type SignInField = "email" | "password";
export type FailedSignInStatus = {
	isLoading: false;
	success: false;
	errors: Array<Errors.FormValidationError<SignInField>> | Errors.GenericError;
};

export type SignInStatusResponse = SuccessfulSignInStatus | FailedSignInStatus;
export type SignInStatus = InitialSignInStatus | LoadingSignInStatus | SignInStatusResponse;

export class SignIn extends Base {
	public initialStatus: SignInStatus = {
		isLoading: false
	};

	@observable
	public status = _.cloneDeep(this.initialStatus);

	public initialState: SignInState = {
		email: "",
		password: ""
	};

	@observable
	public state = _.cloneDeep(this.initialState);

	@action.bound
	public setEmail(email: string) {
		this.state.email = email;
	}

	@action.bound
	public setPassword(password: string) {
		this.state.password = password;
	}

	public validate(): Array<Errors.FormValidationError<SignInField>> {
		const { email, password } = this.state;
		const errors: Array<Errors.FormValidationError<SignInField>> = [];

		if (email.trim().length === 0)
			errors.push(
				Errors.formValidationError<SignInField>("email", "EMPTY_FIELD", "Campo email no puede estar vacío.")
			);

		if (password.trim().length === 0)
			errors.push(
				Errors.formValidationError<SignInField>(
					"password",
					"EMPTY_FIELD",
					"Campo password no puede estar vacío."
				)
			);

		return errors;
	}

	@bind
	public async signIn(): Promise<SignInStatusResponse> {
		const { email, password } = this.state;
		const { userStore } = this;

		let newStatus: SignInStatus = {
			isLoading: true
		};

		this.setStatus(newStatus);

		const errors = this.validate();

		if (errors.length > 0)
			return Promise.resolve(
				this.setStatus({
					isLoading: false,
					success: false,
					errors
				})
			);

		const request: API.SignInRequest = {
			email,
			password
		};

		const apiResponse = await API.User.signIn(request);

		if (apiResponse.success) {
			const { token } = apiResponse;
			const user = userStore.saveUser(apiResponse.user);

			userStore.setCurrentUser(user);
			userStore.setToken(token);

			newStatus = {
				isLoading: false,
				success: true,
				token,
				user
			};
		} else
			newStatus = {
				isLoading: false,
				success: false,
				errors: apiResponse.error
			};

		return this.setStatus(newStatus);
	}

	@action
	protected setStatus<T extends SignInStatus>(status: T): T {
		return (this.status = status);
	}
}
